//#define	DEBUG

/*
	RMEEp02
	E24系列基準の抵抗計（Arduino単体）
	(C) 2022 JH4VAJ
*/

#define	TITLE		"RMEEp02"
#define	VERSION		"0.9"
#define COPYRIGHT	"(C) 2022 JH4VAJ"

#define	RREF	1.0e3						/* 基準抵抗の値 */


const float e24[] =	 {1.0, 1.1, 1.2, 1.3, 1.5, 1.6, 1.8,
					  2.0, 2.2, 2.4, 2.7,
					  3.0, 3.3, 3.6, 3.9,
					  4.3, 4.7, 5.1, 5.6,
					  6.2, 6.8, 7.5, 8.2, 9.1
					  };


// the setup function runs once when you press reset or power the board
void setup() {
	Serial.begin(9600);
}


void measure_resistance() {
	long	val_x, val_diff;
	float	r;

	const long	val_ref = 1024;		// リファレンス値

	val_x = analogRead(A0);				// 測定値

	val_diff = val_ref - val_x;

#ifdef DEBUG
	Serial.print("val_x: ");
	Serial.print(val_x);
	Serial.print("	val_dif: ");
	Serial.println(val_diff);
#endif

	r = (((float)val_x / (float)val_diff) * RREF);
	print_result(r);
}


void print_result(float r) {
	char	str_buf[10];			// 表示文字列バッファ
	char	*str_kM;				// 接頭辞
	float	kM;						// 接頭辞に合わせた倍率
	float	match_e24_kM;			// マッチしたE24系列の値（接頭辞考慮）
	float	multi;					// E24系列の基数に掛け合わせる倍率
	float	diff_0, diff_1;			// E24系列マッチ判定用
	float	diff_kM;				// E24系列からの差分（接頭辞考慮）
	float	diff_cent;				// E24系列からの差分（百分率）

	const int digi_mes = 5;			// 測定結果表示桁数
	int		n_digi;					// 整数部の表示桁数

	const float	r_thsld = (e24[0] + (e24[(sizeof(e24) / sizeof(float)) - 1] / 10.0)) / 2.0;
									// 桁上げ判定閾値: (1.0 + 0.91) / 2.0）= 0.955
	
	int		i;


	if (r > (r_thsld * 1.0e6)) {	// 接頭辞（k, M）判定
		str_kM = "M";
		kM = 1.0e6;
	}
	else if (r > (r_thsld * 1.0e3)) {
		str_kM = "k";
		kM = 1.0e3;
	}
	else {
		str_kM = "";
		kM = 1.0;
	}

	if (r < (r_thsld * 1.0)) {		// 乗数と整数部の表示桁数判定
		multi = 0.1;
		n_digi = 1;
	}
	else if (r < (r_thsld * 10.0)) {
		multi = 1.0;
		n_digi = 1;
	}
	else if (r < (r_thsld * 100.0)) {
		multi = 10.0;
		n_digi = 2;
	}
	else if (r < (r_thsld * 1.0e3)) {
		multi = 100.0;
		n_digi = 3;
	}
	else if (r < (r_thsld * 10.0e3)) {
		multi = 1.0e3;
		n_digi = 1;
	}
	else if (r < (r_thsld * 100.0e3)) {
		multi = 10.0e3;
		n_digi = 2;
	}
	else if (r < (r_thsld * 1.0e6)) {
		multi = 100.0e3;
		n_digi = 3;
	}
	else if (r < (r_thsld * 10.0e6)) {
		multi = 1.0e6;
		n_digi = 1;
	}
	else if (r < (r_thsld * 100.0e6)) {
		multi = 10.0e6;
		n_digi = 2;
	}
	else {
		multi = 100.0e6;
		n_digi = 3;
	}

	/* E24系列判定 */
	diff_0 = 1.0e30;		// ダミーの最大値（初期値）

	for (i = 0; i < (sizeof(e24) / sizeof(float)); i++) {	
		diff_1 = r - (e24[i] * multi);
		if (abs(diff_0) < abs(diff_1)) {		// E24系列からの差分が最小のものを見つける
			break;
		}
		else {
			diff_0 = diff_1;
		}
	}

	match_e24_kM = e24[i - 1] * (multi / kM);

	/* 表示 */
	
	/* マッチするE24系列の値 */
	if (n_digi == 1) {				// 整数部が一桁なら小数点以下は一桁（e.g. 4.7kΩ）
		dtostrf(match_e24_kM, 3, 1, str_buf);
	}
	else {							// それ以外は小数点以下はなし（e.g. 33kΩ）
		dtostrf(match_e24_kM, n_digi, 0, str_buf);
	}
	Serial.print("E24_");
	Serial.print(str_buf);
	Serial.print(str_kM);
	Serial.print("Ω");

	/* 測定値 */
	dtostrf((r / kM), digi_mes, (digi_mes - 1 - n_digi), str_buf);
	Serial.print(" ");
	Serial.print(str_buf);
	Serial.print(str_kM);
	Serial.print("Ω");

	/* E24系列からの差分 */
	diff_kM = diff_0 / kM;
	dtostrf(diff_kM, 7, 3, str_buf);
	Serial.print(str_buf);
	Serial.print(str_kM);
	Serial.print("Ω");

	/* 誤差 */
	diff_cent = (diff_kM / match_e24_kM) * 100.0;
	dtostrf(diff_cent, 6, 2, str_buf);
	Serial.print(str_buf);
	Serial.print("%");

	Serial.println("");
}




// the loop function runs over and over again forever
void loop() {
	measure_resistance();

	delay(1000);
}
